from Products.CMFCore.utils import getToolByName

# Properties are defined here, because if they are defined in propertiestool.xml,
# all properties are re-set the their initial state if you reinstall product
# in the quickinstaller.


_PROPERTIES = [ dict(name='header_allow_inheritance', type_='boolean', value=True),
                dict(name='header_width', type_='int', value=0),
                dict(name='header_height', type_='int', value=0)]

def setupProperties(context):
    if not context.readDataFile('raptus.header_setupProperties.txt'):
        return

    portal = context.getSite()

    props = getToolByName(portal, 'portal_properties').site_properties
    for prop in _PROPERTIES:
        if not props.hasProperty(prop['name']):
            props.manage_addProperty(prop['name'], prop['value'], prop['type_'])


from zope import schema
from zope.interface import Interface

from zope.app.container.constraints import contains
from zope.app.container.constraints import containers

from raptus.header import headerMessageFactory as _

class IHeader(Interface):
    """This header will display all contained images on top of your site"""
    
    # -*- schema definition goes here -*-



"""Common configuration constants
"""

from AccessControl import ModuleSecurityInfo
from Products.CMFCore.permissions import setDefaultRoles

PROJECTNAME = 'raptus.header'

security = ModuleSecurityInfo('raptus.header.config')

security.declarePublic('ADD_PERMISSION')
ADD_PERMISSION = {}

ADD_PERMISSION['Header'] = 'raptus.header: Add Header'
setDefaultRoles(ADD_PERMISSION['Header'], ('Manager','Contributor',))


from zope import component
from zope.interface import implements
from random import *

from plone.memoize.instance import memoize
from plone.app.layout.viewlets.common import ViewletBase

from Products.ATContentTypes.interface.image import IATImage
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot

from raptus.header import headerMessageFactory as _
from raptus.header.interfaces import IHeader

from zope.annotation import IAnnotations

from Acquisition import aq_inner, aq_parent

ANNOTATIONS_KEY_DISABLED = 'raptus.header.disabled'
ANNOTATIONS_KEY_ENABLED = 'raptus.header.enabled'

class HeaderViewlet(ViewletBase):
    index = ViewPageTemplateFile('header.pt')
    _info = ViewPageTemplateFile('info.pt')
    
    hasImage = False
    src = ''
    description = ''
    title = ''
    manage = None
    disabled = None
    enabled = None
    info = None

    def __init__(self, context, request, view, manager=None):
        super(HeaderViewlet, self).__init__(context, request, view, manager)
        self.mship = getToolByName(self.context, 'portal_membership')
        self.catalog = getToolByName(self.context, 'portal_catalog')
        self.props = getToolByName(self.context, 'portal_properties').site_properties
        if self.request.get('enable_header', False) or self.request.get('disable_header', False) or self.request.get('inherit_header', False):
            if not self.mship.checkPermission(permissions.ModifyPortalContent, self.context):
                return
            annotations = IAnnotations(context)
            if self.request.get('enable_header', False):
                if annotations.has_key(ANNOTATIONS_KEY_DISABLED):
                    del annotations[ANNOTATIONS_KEY_DISABLED]
                annotations[ANNOTATIONS_KEY_ENABLED] = True
            elif self.request.get('disable_header', False):
                if annotations.has_key(ANNOTATIONS_KEY_ENABLED):
                    del annotations[ANNOTATIONS_KEY_ENABLED]
                annotations[ANNOTATIONS_KEY_DISABLED] = True
            elif self.request.get('inherit_header', False):
                if annotations.has_key(ANNOTATIONS_KEY_DISABLED):
                    del annotations[ANNOTATIONS_KEY_DISABLED]
                if annotations.has_key(ANNOTATIONS_KEY_ENABLED):
                    del annotations[ANNOTATIONS_KEY_ENABLED]

    @memoize
    def info(self):
        header = self.header()
        if not header or (hasattr(self.context, 'isTemporary') and self.context.isTemporary()) or not self.mship.checkPermission(permissions.ModifyPortalContent, self.context):
            return
        annotations = component.queryAdapter(self.context,interface=IAnnotations)
        if annotations is None:
            return
        self.enable = False
        self.disable = False
        self.inherit = annotations.has_key(ANNOTATIONS_KEY_DISABLED) or annotations.has_key(ANNOTATIONS_KEY_ENABLED)
        if self.disabled:
            self.enable = self.mship.checkPermission(permissions.ModifyPortalContent, self.context)
        else:
            self.disable = self.mship.checkPermission(permissions.ModifyPortalContent, self.context)
        return self._info()

    @memoize
    def header(self):
        self.disabled = None
        self.enabled = None

        parent = aq_inner(self.context)
        while True:
            try:
                if not self.enabled and not self.disabled and IAnnotations(parent).get(ANNOTATIONS_KEY_DISABLED, False):
                    self.disabled = parent
                if not self.disabled and not self.enabled and IAnnotations(parent).get(ANNOTATIONS_KEY_ENABLED, False):
                    self.enabled = parent
            except:
                pass
            brain = self.catalog(object_provides=IHeader.__identifier__,
                                 path={'query': '/'.join(parent.getPhysicalPath()),'depth': 1})
            if len(brain) or IPloneSiteRoot.providedBy(parent) or not self.props.getProperty('header_allow_inheritance', True):
                break
            else:
                parent = aq_parent(parent)
        if not len(brain):
            return
        
        """take the first header-element
        """
        return brain[0]
    
    def update(self):
        self.hasImage = False
        header = self.header()
        if not header:
            return
        images = self.catalog(object_provides=IATImage.__identifier__,
                              path={'query': header.getPath(),'depth': 1})

        if not len(images):
            return

        image = choice(images)
        obj = image.getObject()
        scales = component.getMultiAdapter((obj, obj.REQUEST), name='images')
        w = self.props.getProperty('header_width', 0)
        h = self.props.getProperty('header_height', 0)
        scale = scales.scale('image',
                             width=(w and w or 1000000),
                             height=(h and h or 1000000))
        if scale is None:
            return

        self.image = scale.tag()
        self.description = image['Description']
        self.title = image['Title']
        """ if article core exist, so this will load the manage box
        """
        
        if self.disabled:
            return
        
        self.hasImage = True

        try:
            from raptus.article.core import interfaces
            if not hasattr(self.context, 'raptus_article_macros'):
                return
            manageable = interfaces.IManageable(self.context)
            self.manage = manageable.getList([image]).pop()
        except ImportError:
            pass



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

version = '2.0b10'

tests_require=['zope.testing']

setup(name='raptus.header',
      version=version,
      description="Provides a new folderish content type which may contain multiple images which are then displayed by the provided viewlet.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='',
      author='Raptus AG',
      author_email='dev@raptus.com',
      url='http://raptus.com',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['raptus', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'plone.app.imaging',
                        # -*- Extra requirements: -*-
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'raptus.header.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*- 
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



"""Definition of the Header content type
"""

from zope.interface import implements, directlyProvides

from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from raptus.header import headerMessageFactory as _
from raptus.header.interfaces import IHeader
from raptus.header.config import PROJECTNAME

HeaderSchema = folder.ATFolderSchema.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

))

"""hidden not used schema fields"""
for field in ('excludeFromNav','creators','allowDiscussion','contributors','location','language', 'nextPreviousEnabled', 'rights' ):
    if HeaderSchema.has_key(field):
        HeaderSchema[field].widget.visible = {'edit': 'invisible', 'view': 'invisible'}

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

HeaderSchema['title'].storage = atapi.AnnotationStorage()
HeaderSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    HeaderSchema,
    folderish=True,
    moveDiscussion=False
)

class Header(folder.ATFolder):
    """This header will display all contained images on top of your site"""
    implements(IHeader)
    meta_type = "Header"
    schema = HeaderSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    
    # -*- Your ATSchema to Python Property Bridges Here ... -*-

atapi.registerType(Header, PROJECTNAME)
